from invenio_records_resources.services import RecordLink
from invenio_records_resources.services import (
    RecordServiceConfig as InvenioRecordServiceConfig,
)
from invenio_records_resources.services import pagination_links

from nr_documents_records.records.api import NRDocumentRecord
from nr_documents_records.services.permissions import NRDocumentRecordPermissionPolicy
from nr_documents_records.services.schema import NRDocumentSchema
from nr_documents_records.services.search import NRDocumentRecordSearchOptions


class NRDocumentRecordServiceConfig(InvenioRecordServiceConfig):
    """NRDocumentRecord service config."""

    permission_policy_cls = NRDocumentRecordPermissionPolicy
    schema = NRDocumentSchema
    search = NRDocumentRecordSearchOptions
    record_cls = NRDocumentRecord

    components = [*InvenioRecordServiceConfig.components]

    model = "nr_documents_records"

    @property
    def links_item(self):
        return {
            "self": RecordLink("/nr_documents_records/{id}"),
        }

    links_search = pagination_links("/nr_documents_records/{?args*}")



from invenio_records_resources.services import SearchOptions as InvenioSearchOptions

from . import facets


def _(x):
    """Identity function for string extraction."""
    return x


class NRDocumentRecordSearchOptions(InvenioSearchOptions):
    """NRDocumentRecord search options."""

    facets = {
        "metadata_extent": facets.metadata_extent,
        "metadata_dateIssued": facets.metadata_dateIssued,
        "metadata_title": facets.metadata_title,
        "metadata_dateAvailable": facets.metadata_dateAvailable,
        "metadata_dateModified": facets.metadata_dateModified,
        "metadata_abstract": facets.metadata_abstract,
        "metadata_methods": facets.metadata_methods,
        "metadata_technicalInfo": facets.metadata_technicalInfo,
        "metadata_version": facets.metadata_version,
        "metadata_accessibility": facets.metadata_accessibility,
        "metadata_externalLocation_externalLocationURL": facets.metadata_externalLocation_externalLocationURL,
        "metadata_originalRecord": facets.metadata_originalRecord,
        "metadata_collections": facets.metadata_collections,
        "metadata_additionalTitles_title": facets.metadata_additionalTitles_title,
        "metadata_additionalTitles_titleType": facets.metadata_additionalTitles_titleType,
        "metadata_additionalTitles": facets.metadata_additionalTitles,
        "metadata_creators_fullName": facets.metadata_creators_fullName,
        "metadata_creators_nameType": facets.metadata_creators_nameType,
        "metadata_creators_authorityIdentifiers_identifier": facets.metadata_creators_authorityIdentifiers_identifier,
        "metadata_creators_authorityIdentifiers_scheme": facets.metadata_creators_authorityIdentifiers_scheme,
        "metadata_creators_authorityIdentifiers": facets.metadata_creators_authorityIdentifiers,
        "metadata_creators_affiliations": facets.metadata_creators_affiliations,
        "metadata_creators": facets.metadata_creators,
        "metadata_contributors_fullName": facets.metadata_contributors_fullName,
        "metadata_contributors_nameType": facets.metadata_contributors_nameType,
        "metadata_contributors_authorityIdentifiers_identifier": facets.metadata_contributors_authorityIdentifiers_identifier,
        "metadata_contributors_authorityIdentifiers_scheme": facets.metadata_contributors_authorityIdentifiers_scheme,
        "metadata_contributors_authorityIdentifiers": facets.metadata_contributors_authorityIdentifiers,
        "metadata_contributors_affiliations": facets.metadata_contributors_affiliations,
        "metadata_contributors": facets.metadata_contributors,
        "metadata_subjects_subjectScheme": facets.metadata_subjects_subjectScheme,
        "metadata_subjects_subject": facets.metadata_subjects_subject,
        "metadata_subjects_valueURI": facets.metadata_subjects_valueURI,
        "metadata_subjects_classificationCode": facets.metadata_subjects_classificationCode,
        "metadata_subjects": facets.metadata_subjects,
        "metadata_subjectCategories": facets.metadata_subjectCategories,
        "metadata_languages": facets.metadata_languages,
        "metadata_rights": facets.metadata_rights,
        "metadata_relatedItems_itemURL": facets.metadata_relatedItems_itemURL,
        "metadata_relatedItems_itemYear": facets.metadata_relatedItems_itemYear,
        "metadata_relatedItems_itemVolume": facets.metadata_relatedItems_itemVolume,
        "metadata_relatedItems_itemIssue": facets.metadata_relatedItems_itemIssue,
        "metadata_relatedItems_itemStartPage": facets.metadata_relatedItems_itemStartPage,
        "metadata_relatedItems_itemEndPage": facets.metadata_relatedItems_itemEndPage,
        "metadata_relatedItems_itemPublisher": facets.metadata_relatedItems_itemPublisher,
        "metadata_relatedItems_itemCreators_fullName": facets.metadata_relatedItems_itemCreators_fullName,
        "metadata_relatedItems_itemCreators_nameType": facets.metadata_relatedItems_itemCreators_nameType,
        "metadata_relatedItems_itemCreators_authorityIdentifiers_identifier": facets.metadata_relatedItems_itemCreators_authorityIdentifiers_identifier,
        "metadata_relatedItems_itemCreators_authorityIdentifiers_scheme": facets.metadata_relatedItems_itemCreators_authorityIdentifiers_scheme,
        "metadata_relatedItems_itemCreators_authorityIdentifiers": facets.metadata_relatedItems_itemCreators_authorityIdentifiers,
        "metadata_relatedItems_itemCreators_affiliations": facets.metadata_relatedItems_itemCreators_affiliations,
        "metadata_relatedItems_itemCreators": facets.metadata_relatedItems_itemCreators,
        "metadata_relatedItems_itemContributors_fullName": facets.metadata_relatedItems_itemContributors_fullName,
        "metadata_relatedItems_itemContributors_nameType": facets.metadata_relatedItems_itemContributors_nameType,
        "metadata_relatedItems_itemContributors_authorityIdentifiers_identifier": facets.metadata_relatedItems_itemContributors_authorityIdentifiers_identifier,
        "metadata_relatedItems_itemContributors_authorityIdentifiers_scheme": facets.metadata_relatedItems_itemContributors_authorityIdentifiers_scheme,
        "metadata_relatedItems_itemContributors_authorityIdentifiers": facets.metadata_relatedItems_itemContributors_authorityIdentifiers,
        "metadata_relatedItems_itemContributors_affiliations": facets.metadata_relatedItems_itemContributors_affiliations,
        "metadata_relatedItems_itemContributors": facets.metadata_relatedItems_itemContributors,
        "metadata_relatedItems_itemPIDs_identifier": facets.metadata_relatedItems_itemPIDs_identifier,
        "metadata_relatedItems_itemPIDs_scheme": facets.metadata_relatedItems_itemPIDs_scheme,
        "metadata_relatedItems_itemPIDs": facets.metadata_relatedItems_itemPIDs,
        "metadata_relatedItems": facets.metadata_relatedItems,
        "metadata_fundingReferences_projectID": facets.metadata_fundingReferences_projectID,
        "metadata_fundingReferences": facets.metadata_fundingReferences,
        "metadata_geoLocations_geoLocationPlace": facets.metadata_geoLocations_geoLocationPlace,
        "metadata_geoLocations_geoLocationPoint_pointLongitude": facets.metadata_geoLocations_geoLocationPoint_pointLongitude,
        "metadata_geoLocations_geoLocationPoint_pointLatitude": facets.metadata_geoLocations_geoLocationPoint_pointLatitude,
        "metadata_geoLocations": facets.metadata_geoLocations,
        "metadata_series_seriesTitle": facets.metadata_series_seriesTitle,
        "metadata_series_seriesVolume": facets.metadata_series_seriesVolume,
        "metadata_series": facets.metadata_series,
        "metadata_objectIdentifiers_identifier": facets.metadata_objectIdentifiers_identifier,
        "metadata_objectIdentifiers_scheme": facets.metadata_objectIdentifiers_scheme,
        "metadata_objectIdentifiers": facets.metadata_objectIdentifiers,
        "metadata_systemIdentifiers_identifier": facets.metadata_systemIdentifiers_identifier,
        "metadata_systemIdentifiers_scheme": facets.metadata_systemIdentifiers_scheme,
        "metadata_systemIdentifiers": facets.metadata_systemIdentifiers,
        "metadata_events_eventDate": facets.metadata_events_eventDate,
        "metadata_events_eventLocation_place": facets.metadata_events_eventLocation_place,
        "metadata_events": facets.metadata_events,
        "_id": facets._id,
        "created": facets.created,
        "updated": facets.updated,
        "_schema": facets._schema,
    }
    sort_options = {
        **InvenioSearchOptions.sort_options,
        "bestmatch": dict(
            title=_("Best match"),
            fields=["_score"],  # ES defaults to desc on `_score` field
        ),
        "newest": dict(
            title=_("Newest"),
            fields=["-created"],
        ),
        "oldest": dict(
            title=_("Oldest"),
            fields=["created"],
        ),
    }



import marshmallow as ma
import marshmallow.fields as ma_fields
import marshmallow.validate as ma_valid
from invenio_records_resources.services.records.schema import BaseRecordSchema
from invenio_records_resources.services.records.schema import (
    BaseRecordSchema as InvenioBaseRecordSchema,
)
from marshmallow import ValidationError
from marshmallow import validates as ma_validates
from nr_vocabularies.services.schema import NRVocabularySchema
from oarepo_vocabularies.services.schema import VocabularyRelationField

from nr_documents_records.records.api import NRDocumentRecord
from nr_documents_records.services.multilingual_schema import MultilingualSchema


class NRDocumentRecordMetadataSchema(
    ma.Schema,
):
    """NRDocumentRecordMetadataSchema schema."""

    extent = ma_fields.String()

    dateIssued = ma_fields.String(required=True)

    title = ma_fields.List(
        ma_fields.Nested(lambda: MultilingualSchema(required=True), required=True)
    )

    resourceType = VocabularyRelationField(
        NRVocabularySchema,
        required=True,
        related_field=NRDocumentRecord.relations.resourceType,
        many=False,
    )

    dateAvailable = ma_fields.String()

    dateModified = ma_fields.String()

    abstract = ma_fields.List(ma_fields.Nested(lambda: MultilingualSchema()))

    methods = ma_fields.List(ma_fields.Nested(lambda: MultilingualSchema()))

    technicalInfo = ma_fields.List(ma_fields.Nested(lambda: MultilingualSchema()))

    accessRights = VocabularyRelationField(
        NRVocabularySchema,
        related_field=NRDocumentRecord.relations.accessRights,
        many=False,
    )

    version = ma_fields.String()

    accessibility = ma_fields.List(ma_fields.Nested(lambda: MultilingualSchema()))

    externalLocation = ma_fields.Nested(ExternalLocationSchema)

    originalRecord = ma_fields.String()

    collections = ma_fields.List(ma_fields.String())

    additionalTitles = ma_fields.List(ma_fields.Nested(AdditionalTitlesSchema))

    creators = ma_fields.List(ma_fields.Nested(CreatorsSchema))

    contributors = ma_fields.List(ma_fields.Nested(ContributorsSchema))

    subjects = ma_fields.List(ma_fields.Nested(SubjectsSchema))

    publishers = ma_fields.List(ma_fields.String())

    subjectCategories = VocabularyRelationField(
        NRVocabularySchema,
        related_field=NRDocumentRecord.relations.subjectCategories,
        many=True,
    )

    languages = VocabularyRelationField(
        NRVocabularySchema,
        required=True,
        related_field=NRDocumentRecord.relations.languages,
        many=True,
    )

    notes = ma_fields.List(ma_fields.String())

    rights = VocabularyRelationField(
        NRVocabularySchema, related_field=NRDocumentRecord.relations.rights, many=True
    )

    relatedItems = ma_fields.List(ma_fields.Nested(RelatedItemsSchema))

    fundingReferences = ma_fields.List(ma_fields.Nested(FundingReferencesSchema))

    geoLocations = ma_fields.List(ma_fields.Nested(GeoLocationsSchema))

    series = ma_fields.List(ma_fields.Nested(SeriesSchema))

    objectIdentifiers = ma_fields.List(ma_fields.Nested(ObjectIdentifiersSchema))

    systemIdentifiers = ma_fields.List(ma_fields.Nested(SystemIdentifiersSchema))

    events = ma_fields.List(ma_fields.Nested(EventsSchema))


class NRDocumentSchema(
    BaseRecordSchema,
):
    """NRDocumentSchema schema."""

    metadata = ma_fields.Nested(NRDocumentRecordMetadataSchema)

    created = ma_fields.Date(dump_only=True)

    updated = ma_fields.Date(dump_only=True)



from nr_documents_records import config as config


class NRDocumentsRecordsExt(object):
    """extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        self.resource = None
        self.service = None
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)
        self.init_resource(app)
        app.extensions["nr-documents-records"] = self

    def init_resource(self, app):
        """Initialize vocabulary resources."""
        self.service = app.config["NR_DOCUMENTS_RECORDS_SERVICE_CLASS"](
            config=app.config["NR_DOCUMENTS_RECORDS_SERVICE_CONFIG"](),
        )
        self.resource = app.config["NR_DOCUMENTS_RECORDS_RESOURCE_CLASS"](
            service=self.service,
            config=app.config["NR_DOCUMENTS_RECORDS_RESOURCE_CONFIG"](),
        )

    def init_config(self, app):
        """Initialize configuration."""
        app.config.setdefault(
            "NR_DOCUMENTS_RECORDS_RESOURCE_CONFIG",
            config.NR_DOCUMENTS_RECORDS_RESOURCE_CONFIG,
        )
        app.config.setdefault(
            "NR_DOCUMENTS_RECORDS_RESOURCE_CLASS",
            config.NR_DOCUMENTS_RECORDS_RESOURCE_CLASS,
        )
        app.config.setdefault(
            "NR_DOCUMENTS_RECORDS_SERVICE_CONFIG",
            config.NR_DOCUMENTS_RECORDS_SERVICE_CONFIG,
        )
        app.config.setdefault(
            "NR_DOCUMENTS_RECORDS_SERVICE_CLASS",
            config.NR_DOCUMENTS_RECORDS_SERVICE_CLASS,
        )



from invenio_records_resources.resources import (
    RecordResourceConfig as InvenioRecordResourceConfig,
)


class NRDocumentRecordResourceConfig(InvenioRecordResourceConfig):
    """NRDocumentRecord resource config."""

    blueprint_name = "NRDocumentRecord"
    url_prefix = "/nr_documents_records/"



from invenio_records_resources.resources import RecordResource as InvenioRecordResource


class NRDocumentRecordResource(InvenioRecordResource):
    """NRDocumentRecord resource."""

    # here you can for example redefine
    # create_url_rules function to add your own rules



"""Facet definitions."""

from elasticsearch_dsl import Facet
from elasticsearch_dsl.query import Nested
from invenio_records_resources.services.records.facets import TermsFacet


class NestedLabeledFacet(Facet):
    agg_type = "nested"

    def __init__(self, path, nested_facet, label=""):
        self._path = path
        self._inner = nested_facet
        self._label = label
        super(NestedLabeledFacet, self).__init__(
            path=path,
            aggs={
                "inner": nested_facet.get_aggregation(),
            },
        )

    def get_values(self, data, filter_values):
        return self._inner.get_values(data.inner, filter_values)

    def add_filter(self, filter_values):
        inner_q = self._inner.add_filter(filter_values)
        if inner_q:
            return Nested(path=self._path, query=inner_q)

    def get_labelled_values(self, data, filter_values):
        """Get a labelled version of a bucket."""
        try:
            out = data["buckets"]
        except:
            out = []
        return {"buckets": out, "label": str(self._label)}


metadata_extent = TermsFacet(field="metadata.extent")


metadata_dateIssued = TermsFacet(field="metadata.dateIssued")


metadata_title = TermsFacet(field="metadata.title")


metadata_dateAvailable = TermsFacet(field="metadata.dateAvailable")


metadata_dateModified = TermsFacet(field="metadata.dateModified")


metadata_abstract = TermsFacet(field="metadata.abstract")


metadata_methods = TermsFacet(field="metadata.methods")


metadata_technicalInfo = TermsFacet(field="metadata.technicalInfo")


metadata_version = TermsFacet(field="metadata.version")


metadata_accessibility = TermsFacet(field="metadata.accessibility")


metadata_externalLocation_externalLocationURL = TermsFacet(
    field="metadata.externalLocation.externalLocationURL"
)


metadata_originalRecord = TermsFacet(field="metadata.originalRecord")


metadata_collections = TermsFacet(field="metadata.collections")


metadata_additionalTitles_title = TermsFacet(field="metadata.additionalTitles.title")


metadata_additionalTitles_titleType = TermsFacet(
    field="metadata.additionalTitles.titleType"
)


metadata_additionalTitles = TermsFacet(field="metadata.additionalTitles")


metadata_creators_fullName = TermsFacet(field="metadata.creators.fullName")


metadata_creators_nameType = TermsFacet(field="metadata.creators.nameType")


metadata_creators_authorityIdentifiers_identifier = TermsFacet(
    field="metadata.creators.authorityIdentifiers.identifier"
)


metadata_creators_authorityIdentifiers_scheme = TermsFacet(
    field="metadata.creators.authorityIdentifiers.scheme"
)


metadata_creators_authorityIdentifiers = TermsFacet(
    field="metadata.creators.authorityIdentifiers"
)


metadata_creators_affiliations = TermsFacet(field="metadata.creators.affiliations")


metadata_creators = TermsFacet(field="metadata.creators")


metadata_contributors_fullName = TermsFacet(field="metadata.contributors.fullName")


metadata_contributors_nameType = TermsFacet(field="metadata.contributors.nameType")


metadata_contributors_authorityIdentifiers_identifier = TermsFacet(
    field="metadata.contributors.authorityIdentifiers.identifier"
)


metadata_contributors_authorityIdentifiers_scheme = TermsFacet(
    field="metadata.contributors.authorityIdentifiers.scheme"
)


metadata_contributors_authorityIdentifiers = TermsFacet(
    field="metadata.contributors.authorityIdentifiers"
)


metadata_contributors_affiliations = TermsFacet(
    field="metadata.contributors.affiliations"
)


metadata_contributors = TermsFacet(field="metadata.contributors")


metadata_subjects_subjectScheme = TermsFacet(field="metadata.subjects.subjectScheme")


metadata_subjects_subject = TermsFacet(field="metadata.subjects.subject")


metadata_subjects_valueURI = TermsFacet(field="metadata.subjects.valueURI")


metadata_subjects_classificationCode = TermsFacet(
    field="metadata.subjects.classificationCode"
)


metadata_subjects = TermsFacet(field="metadata.subjects")


metadata_subjectCategories = TermsFacet(field="metadata.subjectCategories")


metadata_languages = TermsFacet(field="metadata.languages")


metadata_rights = TermsFacet(field="metadata.rights")


metadata_relatedItems_itemURL = TermsFacet(field="metadata.relatedItems.itemURL")


metadata_relatedItems_itemYear = TermsFacet(field="metadata.relatedItems.itemYear")


metadata_relatedItems_itemVolume = TermsFacet(field="metadata.relatedItems.itemVolume")


metadata_relatedItems_itemIssue = TermsFacet(field="metadata.relatedItems.itemIssue")


metadata_relatedItems_itemStartPage = TermsFacet(
    field="metadata.relatedItems.itemStartPage"
)


metadata_relatedItems_itemEndPage = TermsFacet(
    field="metadata.relatedItems.itemEndPage"
)


metadata_relatedItems_itemPublisher = TermsFacet(
    field="metadata.relatedItems.itemPublisher"
)


metadata_relatedItems_itemCreators_fullName = TermsFacet(
    field="metadata.relatedItems.itemCreators.fullName"
)


metadata_relatedItems_itemCreators_nameType = TermsFacet(
    field="metadata.relatedItems.itemCreators.nameType"
)


metadata_relatedItems_itemCreators_authorityIdentifiers_identifier = TermsFacet(
    field="metadata.relatedItems.itemCreators.authorityIdentifiers.identifier"
)


metadata_relatedItems_itemCreators_authorityIdentifiers_scheme = TermsFacet(
    field="metadata.relatedItems.itemCreators.authorityIdentifiers.scheme"
)


metadata_relatedItems_itemCreators_authorityIdentifiers = TermsFacet(
    field="metadata.relatedItems.itemCreators.authorityIdentifiers"
)


metadata_relatedItems_itemCreators_affiliations = TermsFacet(
    field="metadata.relatedItems.itemCreators.affiliations"
)


metadata_relatedItems_itemCreators = TermsFacet(
    field="metadata.relatedItems.itemCreators"
)


metadata_relatedItems_itemContributors_fullName = TermsFacet(
    field="metadata.relatedItems.itemContributors.fullName"
)


metadata_relatedItems_itemContributors_nameType = TermsFacet(
    field="metadata.relatedItems.itemContributors.nameType"
)


metadata_relatedItems_itemContributors_authorityIdentifiers_identifier = TermsFacet(
    field="metadata.relatedItems.itemContributors.authorityIdentifiers.identifier"
)


metadata_relatedItems_itemContributors_authorityIdentifiers_scheme = TermsFacet(
    field="metadata.relatedItems.itemContributors.authorityIdentifiers.scheme"
)


metadata_relatedItems_itemContributors_authorityIdentifiers = TermsFacet(
    field="metadata.relatedItems.itemContributors.authorityIdentifiers"
)


metadata_relatedItems_itemContributors_affiliations = TermsFacet(
    field="metadata.relatedItems.itemContributors.affiliations"
)


metadata_relatedItems_itemContributors = TermsFacet(
    field="metadata.relatedItems.itemContributors"
)


metadata_relatedItems_itemPIDs_identifier = TermsFacet(
    field="metadata.relatedItems.itemPIDs.identifier"
)


metadata_relatedItems_itemPIDs_scheme = TermsFacet(
    field="metadata.relatedItems.itemPIDs.scheme"
)


metadata_relatedItems_itemPIDs = TermsFacet(field="metadata.relatedItems.itemPIDs")


metadata_relatedItems = TermsFacet(field="metadata.relatedItems")


metadata_fundingReferences_projectID = TermsFacet(
    field="metadata.fundingReferences.projectID"
)


metadata_fundingReferences = TermsFacet(field="metadata.fundingReferences")


metadata_geoLocations_geoLocationPlace = TermsFacet(
    field="metadata.geoLocations.geoLocationPlace"
)


metadata_geoLocations_geoLocationPoint_pointLongitude = TermsFacet(
    field="metadata.geoLocations.geoLocationPoint.pointLongitude"
)


metadata_geoLocations_geoLocationPoint_pointLatitude = TermsFacet(
    field="metadata.geoLocations.geoLocationPoint.pointLatitude"
)


metadata_geoLocations = TermsFacet(field="metadata.geoLocations")


metadata_series_seriesTitle = TermsFacet(field="metadata.series.seriesTitle")


metadata_series_seriesVolume = TermsFacet(field="metadata.series.seriesVolume")


metadata_series = TermsFacet(field="metadata.series")


metadata_objectIdentifiers_identifier = TermsFacet(
    field="metadata.objectIdentifiers.identifier"
)


metadata_objectIdentifiers_scheme = TermsFacet(
    field="metadata.objectIdentifiers.scheme"
)


metadata_objectIdentifiers = TermsFacet(field="metadata.objectIdentifiers")


metadata_systemIdentifiers_identifier = TermsFacet(
    field="metadata.systemIdentifiers.identifier"
)


metadata_systemIdentifiers_scheme = TermsFacet(
    field="metadata.systemIdentifiers.scheme"
)


metadata_systemIdentifiers = TermsFacet(field="metadata.systemIdentifiers")


metadata_events_eventDate = TermsFacet(field="metadata.events.eventDate")


metadata_events_eventLocation_place = TermsFacet(
    field="metadata.events.eventLocation.place"
)


metadata_events = TermsFacet(field="metadata.events")


_id = TermsFacet(field="id")


created = TermsFacet(field="created")


updated = TermsFacet(field="updated")


_schema = TermsFacet(field="$schema")



import nr_vocabularies.records.api
import oarepo_vocabularies.records.system_fields.pid_hierarchy_relation
from invenio_pidstore.providers.recordid_v2 import RecordIdProviderV2
from invenio_records.dumpers.relations import RelationDumperExt
from invenio_records.systemfields import ConstantField, RelationsField
from invenio_records_resources.records.api import Record as InvenioBaseRecord
from invenio_records_resources.records.systemfields import IndexField
from invenio_records_resources.records.systemfields.pid import PIDField, PIDFieldContext

from nr_documents_records.records.dumper import NRDocumentRecordDumper
from nr_documents_records.records.models import NRDocumentRecordMetadata
from nr_documents_records.records.multilingual_dumper import MultilingualDumper


class NRDocumentRecord(InvenioBaseRecord):
    model_cls = NRDocumentRecordMetadata
    schema = ConstantField("$schema", "local://nr-documents-records-1.0.0.json")
    index = IndexField("nr_documents_records-nr-documents-records-1.0.0")

    pid = PIDField(
        create=True, provider=RecordIdProviderV2, context_cls=PIDFieldContext
    )

    dumper_extensions = [
        RelationDumperExt("relations"),
    ]
    dumper = NRDocumentRecordDumper(extensions=dumper_extensions)

    relations = RelationsField(
        resourceType=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyRelation(
            "metadata.resourceType",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="resourceType-relation",
        ),
        accessRights=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyRelation(
            "metadata.accessRights",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="accessRights-relation",
        ),
        affiliations=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyListRelation(
            "metadata.creators.affiliations",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="affiliations-relation",
        ),
        role=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyRelation(
            "metadata.contributors.role",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="role-relation",
        ),
        affiliations=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyListRelation(
            "metadata.contributors.affiliations",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="affiliations-relation",
        ),
        subjectCategories=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyListRelation(
            "metadata.subjectCategories",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="subjectCategories-relation",
        ),
        languages=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyListRelation(
            "metadata.languages",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="languages-relation",
        ),
        rights=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyListRelation(
            "metadata.rights",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="rights-relation",
        ),
        itemRelationType=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyRelation(
            "metadata.relatedItems.itemRelationType",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="itemRelationType-relation",
        ),
        itemResourceType=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyRelation(
            "metadata.relatedItems.itemResourceType",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="itemResourceType-relation",
        ),
        affiliations=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyListRelation(
            "metadata.relatedItems.itemCreators.affiliations",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="affiliations-relation",
        ),
        role=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyRelation(
            "metadata.relatedItems.itemContributors.role",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="role-relation",
        ),
        affiliations=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyListRelation(
            "metadata.relatedItems.itemContributors.affiliations",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="affiliations-relation",
        ),
        funder=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyRelation(
            "metadata.fundingReferences.funder",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="funder-relation",
        ),
        country=oarepo_vocabularies.records.system_fields.pid_hierarchy_relation.PIDHierarchyRelation(
            "metadata.events.eventLocation.country",
            keys=["id", "title"],
            pid_field=nr_vocabularies.records.api.NRVocabulary.pid.with_type_ctx(
                "hierarchy"
            ),
            cache_key="country-relation",
        ),
    )


NRDocumentRecord.dumper_extensions.append(MultilingualDumper)



from nr_documents_records.resources.config import NRDocumentRecordResourceConfig
from nr_documents_records.resources.resource import NRDocumentRecordResource
from nr_documents_records.services.config import NRDocumentRecordServiceConfig
from nr_documents_records.services.service import NRDocumentRecordService

NR_DOCUMENTS_RECORDS_RESOURCE_CONFIG = NRDocumentRecordResourceConfig
NR_DOCUMENTS_RECORDS_RESOURCE_CLASS = NRDocumentRecordResource
NR_DOCUMENTS_RECORDS_SERVICE_CONFIG = NRDocumentRecordServiceConfig
NR_DOCUMENTS_RECORDS_SERVICE_CLASS = NRDocumentRecordService
